@@ -1,6 +1,6 @@
 import re
 from copy import deepcopy
-from sbg_cwl_upgrader.cwl_utils import CWL
+from sbg_cwl_upgrader.cwl_utils import CWL, DEFAULT_CWL_VERSION
 import os
 import sbg_cwl_upgrader
 from termcolor import colored
@@ -66,17 +66,6 @@ class Input(CWL):
                 self.cwl['inputBinding'] = CommandLineBinding(
                         sbg_draft2=self.cwl['inputBinding']).to_dict()
 
-                # Place a "default: 0" field
-                # for non-file inputs with valueFrom,
-                # then convert to null
-                if ('valueFrom' in self.cwl['inputBinding']
-                        and not self.is_file_input(sbg_draft2) and in_id):
-                    self.cwl['default'] = 0
-                    self.cwl['inputBinding']['valueFrom'] = self.prepend_js(
-                        self.cwl['inputBinding']['valueFrom'],
-                        'if (self == 0){self = null;\ninputs.' +
-                        in_id + ' = null};\n')
-
                 if self.is_array_input(sbg_draft2):
                     # itemSeparator must go with prefix
                     if ('itemSeparator' in self.cwl['inputBinding']
@@ -171,6 +160,10 @@ class OutputBinding(CWL):
                     and len(self.cwl.get('glob', '')) > 2):
                 self.cwl['glob'] = self.cwl['glob'][2:]
 
+            # Handle brace expand in glob
+            if isinstance(self.cwl.get('glob', None), str):
+                self.cwl['glob'] = self.handle_glob_brace(self.cwl.get('glob'))
+
             if ('outputEval' in sbg_draft2
                     and isinstance(sbg_draft2['outputEval'], dict)
                     and 'script' in sbg_draft2['outputEval']):
@@ -228,6 +221,10 @@ class Output(CWL):
 
 
 class CWLToolConverter(CWL):
+
+    def __init__(self, cwl_version=DEFAULT_CWL_VERSION):
+        self.cwl_version = cwl_version
+
     @staticmethod
     def _is_staged_file(sbg_draft2_input):
         if ('sbg:stageInput' in sbg_draft2_input
@@ -264,7 +261,12 @@ class CWLToolConverter(CWL):
         out = []
         for i in draft2_inputs:
             if 'sbg:stageInput' in i and i['sbg:stageInput']:
-                out.append("$(inputs." + str(i['id']).lstrip('#') + ")")
+                out.append(
+                    {
+                        "entry": "$(inputs." + str(i['id']).lstrip('#') + ")",
+                        "writable": i['sbg:stageInput'] == 'copy'
+                    }
+                )
         return out
 
     @staticmethod
@@ -326,6 +328,8 @@ class CWLToolConverter(CWL):
             if isinstance(hint, dict) and 'class' in hint.keys():
                 # DockerRequirement
                 if hint['class'] == 'DockerRequirement':
+                    if 'dockerImageId' in hint:
+                        del hint['dockerImageId']
                     new_requirements.append(deepcopy(hint))
                 # ResourceRequirement
                 elif ((hint['class'] == 'sbg:CPURequirement')
@@ -411,7 +415,7 @@ class CWLToolConverter(CWL):
                 if re.match(r'[^A-Za-z0-9]+', x):
                     break
                 else:
-                    out.append(x)
+                    out += x.split()
             else:
                 raise Exception("Base command type can be either"
                                 " array of Strings or Expressions.")
@@ -457,6 +461,31 @@ class CWLToolConverter(CWL):
                                 " String or Expression.")
         return out
 
+    def _handle_valuefrom_optional_inputs(self, cwl):
+        """
+        Move inputBinding with valueFrom for optional inputs
+        :param cwl: CWL dict
+        :return: CWL dict
+        """
+        for in_id, inp in cwl['inputs'].items():
+            # In sbg:draft-2 valueFrom was always evaluated
+            # In CWL v1 valueFrom is not evaluated if self is null
+            # Move the valueFrom for optional inputs to arguments
+            if ('valueFrom' in inp.get('inputBinding', {})
+                    and not self.is_required_type(inp['type'])
+                    and "loadContents" not in inp.get('inputBinding', {})):
+                new_arg = deepcopy(inp['inputBinding'])
+                if isinstance(new_arg['valueFrom'], str):
+                    new_arg['valueFrom'] = new_arg['valueFrom'].replace(
+                        "self", "inputs.{}".format(in_id)
+                    )
+                if 'arguments' in cwl:
+                    cwl['arguments'].append(new_arg)
+                else:
+                    cwl['arguments'] = [new_arg]
+                del cwl['inputs'][in_id]['inputBinding']
+        return cwl
+
     @staticmethod
     def _handle_stream(stream):
         if isinstance(stream, str):
@@ -495,8 +524,8 @@ class CWLToolConverter(CWL):
     def convert_dict(self, data: dict):
         """Main method for converting draft2 tool to CWL1.0"""
 
-        # Just reuse if tool is already CWLv1.0
-        if data['cwlVersion'] == 'v1.0':
+        # Just reuse if tool is already CWLv1
+        if data.get('cwlVersion') != 'sbg:draft-2':
             return data
 
         new_data = {k: deepcopy(v)
@@ -510,8 +539,8 @@ class CWLToolConverter(CWL):
                                  'sbg:revision', 'sbg:revisionsInfo',
                                  'sbg:createdOn']}
 
-        new_data['cwlVersion'] = 'v1.0'
-        new_data['sbg:appVersion'] = ['v1.0']
+        new_data['cwlVersion'] = self.cwl_version
+        new_data['sbg:appVersion'] = [self.cwl_version]
         if 'stdin' in data:
             new_data['stdin'] = self._handle_stream(data['stdin'])
         if 'stdout' in data:
@@ -554,6 +583,8 @@ class CWLToolConverter(CWL):
                 offset=offset,
                 min_inp_pos=min_inp_pos)
 
+        new_data = self._handle_valuefrom_optional_inputs(new_data)
+
         data['outputs'] = self._cleanup_invalid_inherit_metadata(
             data['inputs'], data['outputs'])
         if 'outputs' in data:
