@@ -2,10 +2,16 @@ from unittest import TestCase
 from unittest.mock import patch
 import io
 import sbg_cwl_upgrader
+from sbg_cwl_upgrader.converter.cwl_converter import CWLConverterFacade
 from sbg_cwl_upgrader.converter.tool import (Input, Output, Expression,
                                              CommandLineBinding,
                                              CWLToolConverter,
                                              OutputBinding, InputRecordField)
+import random
+import subprocess
+import string
+import os
+import sys
 
 
 class TestInput(TestCase):
@@ -221,21 +227,6 @@ class TestInput(TestCase):
         self.assertEqual(cwl1.cwl['inputBinding']['position'], 1)
         self.assertNotIn('required', cwl1.cwl)
 
-    def test_valueFrom_adding_default_0(self):
-        draft2 = {
-            "type": [
-              "string", "null"
-            ],
-            "inputBinding": {
-                "valueFrom": "{return a}"
-            }
-        }
-        cwl1 = Input(draft2, in_id="input")
-        self.assertEqual(cwl1.cwl['default'], 0)
-        self.assertIn('self = null', cwl1.cwl["inputBinding"]['valueFrom'])
-        self.assertIn('inputs.input = null',
-                      cwl1.cwl["inputBinding"]['valueFrom'])
-
     def test_delete_cmd_include(self):
         draft2 = {
             "inputBinding": {
@@ -302,6 +293,37 @@ class TestOutputBinding(TestCase):
         cwl1 = OutputBinding(draft2)
         self.assertEqual(cwl1.cwl['glob'], 'a/b/c')
 
+    @patch('sys.stdout', new_callable=io.StringIO)
+    def test_glob_with_braces(self, mock_stdout):
+        """
+        glob with brace expand should be split into indiviudal components
+        and run corectly on cwltool.
+        :return:
+        """
+
+        v1_file = os.path.join(
+            os.path.dirname(__file__),
+            'mini_tool_cwl1_{}.json'.format(
+                ''.join(random.sample(string.ascii_lowercase, 3))
+            )
+        )
+        d2_file = os.path.join(os.path.dirname(__file__),
+                               'tool_glob_with_braces_d2.cwl')
+        CWLConverterFacade(d2_file,
+                           output=v1_file)
+
+        process = subprocess.Popen(
+            [sys.executable, "-m", "cwltool", v1_file],
+            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+
+        stdout, _ = process.communicate()
+        os.remove(v1_file)
+
+        self.assertEqual(process.returncode, 0)
+        passed = ("BatchText" in str(stdout)) or ("TextBatch" in str(stdout))
+        self.assertTrue(passed)
+        self.assertIn("Converting done.", mock_stdout.getvalue())
+
     def test_glob_expression(self):
         draft2 = {
             "glob": {
@@ -484,7 +506,8 @@ class TestCWLToolConverter(TestCase):
             }
         ]
         cwl1 = self.tool_converter._stage_inputs(draft2)
-        self.assertIn("$(inputs.foo)", cwl1)
+        self.assertEqual("$(inputs.foo)", cwl1[0]["entry"])
+        self.assertTrue(cwl1[0]["writable"])
         self.assertNotIn("$(inputs.bar)", cwl1)
 
     def test_input_position_shift_with_offset(self):
@@ -512,7 +535,10 @@ class TestCWLToolConverter(TestCase):
             }
         ]
 
-        cwl1 = self.tool_converter._handle_inputs(draft2, range(100), 30, -3)
+        cwl1 = self.tool_converter._handle_inputs(draft2,
+                                                  list(range(100)),
+                                                  30,
+                                                  -3)
         # should be 5 + 100 - 30 - (-3) = 78
         self.assertEqual(cwl1['foo']['inputBinding']['position'], 78)
         # should be -3 + 100 - 30 - (-3) = 70
@@ -619,6 +645,35 @@ class TestCWLToolConverter(TestCase):
         self.assertListEqual(sorted(expected_requirements),
                              sorted([req["class"] for req in cwl1]))
 
+    def test_dockerImageId_removed(self):
+        """
+        Test if dockerImageId is removed upon conversion
+        :return:
+        """
+        draft2_hints = [
+            {
+                "class": "DockerRequirement",
+                "dockerPull": "ubuntu",
+                "dockerImageId": "foo"
+            }
+        ]
+        cwl1_expected = [
+            {
+                "class": "DockerRequirement",
+                "dockerPull": "ubuntu"
+            }
+        ]
+        cwl1 = self.tool_converter._handle_requirements(
+            draft2_hints,
+            [],
+            []
+        )
+        for req in cwl1:
+            if req.get('class') == 'DockerRequirement':
+                cwl1_req = req
+
+        self.assertDictEqual(cwl1_req, cwl1_expected[0])
+
     @patch('sys.stdout', new_callable=io.StringIO)
     def test_create_file_requirement(self, mock_stdout):
         """
@@ -712,6 +767,44 @@ class TestCWLToolConverter(TestCase):
                                                      offset=1)
         self.assertEqual(len(cwl1), 4)
 
+    def test_optional_value_from(self):
+        """
+        Test base command items are added to arguments
+        :return:
+        """
+        cwl1 = {
+            "inputs": {
+                "in1": {
+                    "type": "File",
+                    "inputBinding": {
+                        "position": 1,
+                        "valueFrom": 1
+                    }
+                },
+                "in2": {
+                    "type": "File?",
+                    "inputBinding": {
+                        "position": 2,
+                        "valueFrom": 2
+                    }
+                },
+                "in3": {
+                    "type": ["null", "File"],
+                    "inputBinding": {
+                        "position": 3,
+                        "valueFrom": "${ return self}"
+                    }
+                }
+            }
+        }
+        cwl1 = self.tool_converter._handle_valuefrom_optional_inputs(cwl1)
+        self.assertIn("in1", cwl1["inputs"])
+        self.assertEqual(len(cwl1["arguments"]), 2)
+        self.assertNotIn("inputBinding", cwl1["inputs"]["in2"])
+        self.assertNotIn("inputBinding", cwl1["inputs"]["in3"])
+        self.assertEqual("${ return inputs.in3}",
+                         cwl1["arguments"][1]["valueFrom"])
+
     def test_cleanup_inherit_metadata(self):
         """
         Remove inheriting metadata from non-existing input
